/*
 * File:   main.c
 * Author: Kevin Schneier
 *
 */


/* Preprocessor ***************************************************************/


#define FCY 39613750
#define BAUDRATE 115200
#define ADC_BUFFER_SIZE 1024
#define ADC_RATE 1333.33333
//#define ADC_RATE 1323.767753
// By the math, the ADC_RATE should be 1323.767753; but this isn't what it seems like?
#define SORT 12

/* Motor Defines */
#define s1Up() {SDC1 = 163;}
#define s1Down() {SDC1 = 183;}
#define s2Up() {PDC1 = 163;}
#define s2Down() {PDC1 = 183;}
#define s3Up() {SDC2 = 163;}
#define s3Down() {SDC2 = 183;}
#define s4Up() {PDC2 = 163;}
#define s4Down() {PDC2 = 183;}
#define s5Up() {SDC3 = 163;}
#define s5Down() {SDC3 = 183;}
#define s6Up() {PDC3 = 163;}
#define s6Down() {PDC3 = 183;}

// For calibration between touchscreen and screen coordinates
#define TS_MINX 200
#define TS_MINY 120
#define TS_MAXX 900
#define TS_MAXY 900

#define MINPRESSURE 10
#define MAXPRESSURE 1000

// UI-specific definitions
#define noteBoxSize 46
#define spacing 50


#include "xc.h"
#include "fft.h"
#include <stdlib.h>
#include <math.h>
#include <p33FJ64GS606.h>
#include "config.h"

/* Touchscreen */
#include "touchscreen.h"
#include "touchCapability.h"

/* Get extracting function */
#include "fundamental.h"






/* Function Prototypes ********************************************************/

void init_PLL(void);
void init_timer(void);
void init_ADC(void);
void init_PWM(void);
void init_UART(void);
void send_UART(char c);
unsigned char receive_UART(void);
double decode_UART(void);
void init_SPI(void);
void stopMotors(void);
int motorControl(double measured, double target);
int masterControl(uint16_t *fft, int numPoints, double *freqs);
void init_timer2(void);
int * getPoint(void);
void startTuning(void);
void stopTuning(void);
void tuningDisplay(int tuningStringNum, int mode);
void freqPicker(void);
void resetDisplay(void);
void updateDisplayText(int noteRst);
void selectNote(void);
void rangeMapping(int input_x, int input_y);
void updateScreenMemoryArray(void);


/* Global Variables ***********************************************************/

double frequencies[6] = {0};
//double frequencies[6] = {82.41, 110.0, 146.8, 196.0, 246.9, 329.6};
double fundamental = 0;
double last_fundamental = 0;
double peaks[SORT] = {0};
int freq_index = 0;
int ADC_index = 0;
int FFT_index = 0;
int divisor = 8;
int mode = 0;
double threshold = ADC_RATE/ADC_BUFFER_SIZE;


int ADC_values[ADC_BUFFER_SIZE] = {0};                  // Initialize real array
int ADC_imaginary[ADC_BUFFER_SIZE] = {0};               // Initialize imaginary components to 0
uint16_t amplitudes[ADC_BUFFER_SIZE/2] = {0};           // Initialize amplitude array


/* Touchscreen */
unsigned short _width = ILI9340_TFTWIDTH;
unsigned short _height = ILI9340_TFTHEIGHT;
unsigned short cursor_y = 0;
unsigned short cursor_x = 0;
unsigned short textsize = 0;
unsigned short textcolor = 0;
unsigned short textbgcolor = 0;
unsigned short wrap = 0;
unsigned short rotation = 0;

int    ADC_AN3_val=0;
int    ADC_AN2_val=0;    
int  coords[] = {-99,-99,-99}; 
int touchMode = 0;
int counter=0;
int touchReady = 0;
int i;

// Initialize note data
int selectedString = 0; //note selected on display, can be 0-5
int lastSelectedString = 0;

int noteSpacing[6];
int setNotesArray[6]={4, 0, 3, 6, 1, 4}; //A-0, B-1, C-2, D-3, E-4, F-5, G-6
char fullNotesArray[7] = "ABCDEFG";

int isSharp[6] = {0, 0, 0, 0, 0, 0};

//Initialize conversion data
int touchLoc[2];


// Initialize tuning data
int isTuning = 0;


/* Interrupt Service Routines *************************************************/

void __attribute__((__interrupt__, no_auto_psv)) _T1Interrupt(void) {

    if (ADC_index++ >= ADC_BUFFER_SIZE/divisor) {
        IEC0bits.T1IE = 0;                  // Disable Timer 1 interrupt
        ADC_index = 0;
        IEC7bits.ADCP2IE = 1;               // Enable ADC interrupt, we're ready to sample again
        stopMotors();
    }
    
    
    IFS0bits.T1IF = 0;
    IFS7bits.ADCP2IF = 0;                   // Clear ADC Pair 2 Interrupt Flag

}

void __attribute__((__interrupt__, no_auto_psv)) _ADCP2Interrupt(void) {
    
    ADC_values[ADC_index++] = ADCBUF5;
    ADSTATbits.P2RDY = 0;                   // Clear status bit
    if (ADC_index >= ADC_BUFFER_SIZE){
        IEC7bits.ADCP2IE = 0;               // Disable ADC interrupt when buffer is full
        FFT(ADC_values, ADC_imaginary);
        for (FFT_index = 1; FFT_index < ADC_BUFFER_SIZE/2; FFT_index++){
            amplitudes[FFT_index] = FFTAmplitude(ADC_values[FFT_index], ADC_imaginary[FFT_index]);
        }
        divisor = masterControl(amplitudes, ADC_BUFFER_SIZE/2, frequencies);
        ADC_index = 0;
        IEC0bits.T1IE = 1;                  // Enable timer 1 interrupt
    }     
    
    IFS0bits.T1IF = 0;                      // Clear Timer 1 Interrupt Flag
    IFS7bits.ADCP2IF = 0;                   // Clear ADC Pair 2 Interrupt Flag
}

void __attribute__((__interrupt__, no_auto_psv)) _U2RXInterrupt(void) {
    freq_index = freq_index % 6;
    frequencies[freq_index++] = decode_UART();
    if (freq_index == 6 && frequencies[5] > 1){
        updateScreenMemoryArray();
    }
    IFS1bits.U2RXIF = 0;
}

void __attribute__((__interrupt__, no_auto_psv)) _ADCP1Interrupt(void) {

	/* ISR here */

	/* Whenever you read a value (i.e. myVal = ADCBUF2 or myVar = ADCBUF3),
	you have to do the following immediately after:
	ADSTATbits.P1RDY = 0;	*/

    ADC_AN3_val = ADCBUF3;
    ADC_AN2_val = ADCBUF2;
    
    touchReady = 1;

    ADSTATbits.P1RDY = 0;
    
	/* At the end of the ISR: */
	IFS0bits.T2IF = 0;		// Clear Timer 2 interrupt flag
	IFS6bits.ADCP1IF = 0;		// Clear ADC Pair 1 interrupt flag
	
}



/* Main Execution *************************************************************/

int main(void) {

    /* Setup PLL */
    init_PLL();
    init_SPI();
    
    /* Initialize components */
    init_UART();    
    init_PWM();
    init_timer();
    init_ADC();
    
    /* Initialize ADC*/
    init_timer2();
    
    /* Initialize Screen */
    tft_begin();
    
    /* Do stuff */
    resetDisplay();
    selectNote();


    /* Main loop */
    while(1){
        if (frequencies[5] < 1 && frequencies[0] > 1){
            continue;
        }
        if (frequencies[5] < 1){
            if (isTuning == 1){
                stopTuning();
            }
            PTCONbits.PTEN = 0;
        }
        else {
            if (isTuning == 0){
                startTuning();
            }
            PTCONbits.PTEN = 1;
        }
    
    
        getPoint(); 
        rangeMapping(coords[0],coords[1]);
      
        if (touchLoc[0]<0 || touchLoc[0]>320 || touchLoc[1]<0 || touchLoc[1]>240)
            coords[2]=0;
        if (coords[2]>MINPRESSURE && coords[2]<MAXPRESSURE){
            if (isTuning == 0) {
                                //check if any display functions are being pressed (e.g. note boxes, triangles, etc.)
                                //boxes
                if ((touchLoc[1] > 91) && (touchLoc[1] < (91+noteBoxSize))) {
                    for (i=0;i<6;i++){
                        if ((touchLoc[0] > 12 + i*spacing) && (touchLoc[0] < 12+i*spacing+noteBoxSize)){
                            lastSelectedString = selectedString;
                            selectedString = i;
                        }
                    }
                    selectNote();
                } 
  
    //triangles
    if ((touchLoc[0] > 12+selectedString*spacing) && (touchLoc[0] < 12+selectedString*spacing+noteBoxSize)) {
      //up arrow
      if ((touchLoc[1] > 81-noteBoxSize) && (touchLoc[1] < 81)) {
        //tft_fillCircle(touchLoc[0], touchLoc[1], 3, ILI9340_RED);
        switch (selectedString) {
            
            case 0: // low E string
                
                                //add sharps
                  //if not currently on B or E, and not sharp
                if ((setNotesArray[selectedString]==4) && (isSharp[selectedString]==0)){
                    setNotesArray[selectedString]=1;
                }             
                else if ((setNotesArray[selectedString] != 1) && (setNotesArray[selectedString] != 4) && (isSharp[selectedString] == 0)) { 
                  isSharp[selectedString] = 1;
                }
                //increase note selection by 1
                else if (setNotesArray[selectedString] < 6) {
                  setNotesArray[selectedString] ++;
                  isSharp[selectedString] = 0;
                } else { //loop back to beginning of note array
                  setNotesArray[selectedString] = setNotesArray[selectedString] - 6;
                  isSharp[selectedString] = 0;
                }
                
                
                break;
                
            case 1: // A string
                                                //add sharps
                  //if not currently on B or E, and not sharp
                if ((setNotesArray[selectedString]==0) && (isSharp[selectedString]==0)){
                    setNotesArray[selectedString]=5;
                }             
                else if ((setNotesArray[selectedString] != 1) && (setNotesArray[selectedString] != 4) && (isSharp[selectedString] == 0)) { 
                  isSharp[selectedString] = 1;
                }
                //increase note selection by 1
                else if (setNotesArray[selectedString] < 6) {
                  setNotesArray[selectedString] ++;
                  isSharp[selectedString] = 0;
                } else { //loop back to beginning of note array
                  setNotesArray[selectedString] = setNotesArray[selectedString] - 6;
                  isSharp[selectedString] = 0;
                }
                break;
                
            case 2: // D string
                                                //add sharps
                  //if not currently on B or E, and not sharp
                if ((setNotesArray[selectedString]==3) && (isSharp[selectedString]==0)){
                    setNotesArray[selectedString]=0;
                    isSharp[selectedString]=1;
                }             
                else if ((setNotesArray[selectedString] != 1) && (setNotesArray[selectedString] != 4) && (isSharp[selectedString] == 0)) { 
                  isSharp[selectedString] = 1;
                }
                //increase note selection by 1
                else if (setNotesArray[selectedString] < 6) {
                  setNotesArray[selectedString] ++;
                  isSharp[selectedString] = 0;
                } else { //loop back to beginning of note array
                  setNotesArray[selectedString] = setNotesArray[selectedString] - 6;
                  isSharp[selectedString] = 0;
                }
                break;
                
            case 3: // G string
                                                                //add sharps
                  //if not currently on B or E, and not sharp
                if ((setNotesArray[selectedString]==6) && (isSharp[selectedString]==0)){
                    setNotesArray[selectedString]=3;
                    isSharp[selectedString]=1;
                }             
                else if ((setNotesArray[selectedString] != 1) && (setNotesArray[selectedString] != 4) && (isSharp[selectedString] == 0)) { 
                  isSharp[selectedString] = 1;
                }
                //increase note selection by 1
                else if (setNotesArray[selectedString] < 6) {
                  setNotesArray[selectedString] ++;
                  isSharp[selectedString] = 0;
                } else { //loop back to beginning of note array
                  setNotesArray[selectedString] = setNotesArray[selectedString] - 6;
                  isSharp[selectedString] = 0;
                }
                break;
                
            case 4: // B string 
                                                                //add sharps
                  //if not currently on B or E, and not sharp
                if ((setNotesArray[selectedString]==1) && (isSharp[selectedString]==0)){
                    setNotesArray[selectedString]=6;
                    isSharp[selectedString]=1;
                }             
                else if ((setNotesArray[selectedString] != 1) && (setNotesArray[selectedString] != 4) && (isSharp[selectedString] == 0)) { 
                  isSharp[selectedString] = 1;
                }
                //increase note selection by 1
                else if (setNotesArray[selectedString] < 6) {
                  setNotesArray[selectedString] ++;
                  isSharp[selectedString] = 0;
                } else { //loop back to beginning of note array
                  setNotesArray[selectedString] = setNotesArray[selectedString] - 6;
                  isSharp[selectedString] = 0;
                }
                break;
                
            case 5: // high E string
                                                                //add sharps
                  //if not currently on B or E, and not sharp
                if ((setNotesArray[selectedString]==4) && (isSharp[selectedString]==0)){
                    setNotesArray[selectedString]=2;                    
                }             
                else if ((setNotesArray[selectedString] != 1) && (setNotesArray[selectedString] != 4) && (isSharp[selectedString] == 0)) { 
                  isSharp[selectedString] = 1;
                }
                //increase note selection by 1
                else if (setNotesArray[selectedString] < 6) {
                  setNotesArray[selectedString] ++;
                  isSharp[selectedString] = 0;
                } else { //loop back to beginning of note array
                  setNotesArray[selectedString] = setNotesArray[selectedString] - 6;
                  isSharp[selectedString] = 0;
                }
                break;
        }
            
            
       
        updateDisplayText(0);
        
    } 
      // down arrow ****FIX THIS****
      else if ((touchLoc[1] > 91+noteBoxSize+10) && (touchLoc[1] < 91+(2*noteBoxSize)+10)){
        //tft_fillCircle(touchLoc[0], touchLoc[1], 3, ILI9340_GREEN);
  
        switch (selectedString) {
            
            case 0: // low E string
                if ((setNotesArray[selectedString]==1) && (isSharp[selectedString]==0)){
                    setNotesArray[selectedString]=4;
                }  
                else if (isSharp[selectedString] == 1){
                    isSharp[selectedString] = 0;
                } 
                else {

                    if (setNotesArray[selectedString] > 0) {
                      setNotesArray[selectedString] --;
                    } else { //loop back to beginning of note array
                      setNotesArray[selectedString] = setNotesArray[selectedString] + 6;
                    }      

                    if ((setNotesArray[selectedString] != 1) && (setNotesArray[selectedString] != 4)) {
                      isSharp[selectedString] = 1;
                    }
                }
                
                break;
                
            case 1: // A string
                if ((setNotesArray[selectedString]==5) && (isSharp[selectedString]==0)){
                    setNotesArray[selectedString]=0;
                }  
                else if (isSharp[selectedString] == 1){
                    isSharp[selectedString] = 0;
                } 
                else {

                    if (setNotesArray[selectedString] > 0) {
                      setNotesArray[selectedString] --;
                    } else { //loop back to beginning of note array
                      setNotesArray[selectedString] = setNotesArray[selectedString] + 6;
                    }      

                    if ((setNotesArray[selectedString] != 1) && (setNotesArray[selectedString] != 4)) {
                      isSharp[selectedString] = 1;
                    }
                }                
                break;
                
            case 2: // D string
                if ((setNotesArray[selectedString]==0) && (isSharp[selectedString]==1)){
                    setNotesArray[selectedString]=3;
                    isSharp[selectedString]=0;
                }   
                else if (isSharp[selectedString] == 1){
                    isSharp[selectedString] = 0;
                } 
                else {

                    if (setNotesArray[selectedString] > 0) {
                      setNotesArray[selectedString] --;
                    } else { //loop back to beginning of note array
                      setNotesArray[selectedString] = setNotesArray[selectedString] + 6;
                    }      

                    if ((setNotesArray[selectedString] != 1) && (setNotesArray[selectedString] != 4)) {
                      isSharp[selectedString] = 1;
                    }
                }                
                break;
                
            case 3: // G string
                if ((setNotesArray[selectedString]==3) && (isSharp[selectedString]==1)){
                    setNotesArray[selectedString]=6;
                    isSharp[selectedString]=0;
                }  
                else if (isSharp[selectedString] == 1){
                    isSharp[selectedString] = 0;
                } 
                else {

                    if (setNotesArray[selectedString] > 0) {
                      setNotesArray[selectedString] --;
                    } else { //loop back to beginning of note array
                      setNotesArray[selectedString] = setNotesArray[selectedString] + 6;
                    }      

                    if ((setNotesArray[selectedString] != 1) && (setNotesArray[selectedString] != 4)) {
                      isSharp[selectedString] = 1;
                    }
                }                
                break;
                
            case 4: // B string 
                if ((setNotesArray[selectedString]==6) && (isSharp[selectedString]==1)){
                    setNotesArray[selectedString]=1;
                    isSharp[selectedString]=0;
                }
                else if (isSharp[selectedString] == 1){
                    isSharp[selectedString] = 0;
                } 
                else {

                    if (setNotesArray[selectedString] > 0) {
                      setNotesArray[selectedString] --;
                    } else { //loop back to beginning of note array
                      setNotesArray[selectedString] = setNotesArray[selectedString] + 6;
                    }      

                    if ((setNotesArray[selectedString] != 1) && (setNotesArray[selectedString] != 4)) {
                      isSharp[selectedString] = 1;
                    }
                }
                break;
                
            case 5: // high E string
                if ((setNotesArray[selectedString]==2) && (isSharp[selectedString]==0)){
                    setNotesArray[selectedString]=4;                    
                } 
                else if (isSharp[selectedString] == 1){
                    isSharp[selectedString] = 0;
                } 
                else {

                    if (setNotesArray[selectedString] > 0) {
                      setNotesArray[selectedString] --;
                    } else { //loop back to beginning of note array
                      setNotesArray[selectedString] = setNotesArray[selectedString] + 6;
                    }      

                    if ((setNotesArray[selectedString] != 1) && (setNotesArray[selectedString] != 4)) {
                      isSharp[selectedString] = 1;
                    }
                }                
                break;
        }


  
      }
            updateDisplayText(0);

    }
  
    //tune button
    if ((touchLoc[1] > 210) && (touchLoc[1] < 240) && (touchLoc[0] > 235) && (touchLoc[0] < 308)){
        freqPicker();
        startTuning();
    }
  }
  //stop button
        if ((touchLoc[1] > 210) && (touchLoc[1] < 240) && (touchLoc[0] > 25) && (touchLoc[0] < 98)){
            frequencies[0] = 0;
            frequencies[1] = 0;
            frequencies[2] = 0;
            frequencies[3] = 0;
            frequencies[4] = 0;
            frequencies[5] = 0;
            stopTuning();
        }
      
     

      }
    
    }
    return 0;
    
}



/* Function Definitions *******************************************************/

void init_PLL(void) {
    
    /* Configure PLL prescaler, PLL postscaler, PLL divisor */
    PLLFBD = 41;                /* M = PLLFBD + 2 */
    CLKDIVbits.PLLPOST = 0;     /* N1 = 2 */
    CLKDIVbits.PLLPRE = 0;      /* N2 = 2 */

    /* Configure Auxiliary Clock */
    ACLKCONbits.FRCSEL = 1;         /* FRC provides input for Auxiliary PLL (x16) */
    ACLKCONbits.SELACLK = 1;        /* Auxiliary Oscillator provides clock source for PWM & ADC */
    ACLKCONbits.APSTSCLR = 0;       /* Divide Auxiliary clock by 256 */
    ACLKCONbits.ENAPLL = 1;         /* Enable Auxiliary PLL */
    
    /* Wait for clock */
    while(ACLKCONbits.APLLCK != 1);         /* Wait for Auxiliary PLL to Lock */
    
}

void init_timer(void) {
    
    /* Timer 1 Configuration */
    T1CONbits.TON = 0;                  // Turn off Timer 1
    T1CONbits.TCS = 0;
    T1CONbits.TGATE = 0;
    T1CONbits.TCKPS = 0b00;             // Clock prescaler 1:1
    
    /* Timer 1 Value Setting*/
    TMR1 = 0;                           // Reset Timer 1 Value
    PR1 = 39900/4*3;                        // Timer 1 Period Match
    
    /* Timer 1 Interrupt Handling */
    IPC0bits.T1IP = 0x01;               // Set Timer 1 Interrupt Priority
    IFS0bits.T1IF = 0;                  // Clear Timer 1 Interrupt Flag
    IEC0bits.T1IE = 0;                  // Disable Timer 1 Interrupt
    
    /* Timer 1 Activation */
    T1CONbits.TON = 1;                  // Turn on Timer 1
    
}

void init_ADC(void) {	 
    
    /* Set ADC Configuration Bits */
    ADCONbits.ADON = 0;             // Turn ADC Module off
    ADCONbits.FORM = 0;             // Output in integer format
    ADCONbits.SLOWCLK = 0;          // ADC is clocked by primary PLL
    ADCONbits.EIE = 0;              // Interrupt is generated after first conversion is completed
    ADCONbits.ORDER = 0;            // Convert odd channel first
    ADCONbits.SEQSAMP = 0;          // Select simultaneous sampling
    ADCONbits.ADCS = 5;             // ADC clock = FADC/1


    /* Set Trigger Source */
    ADCPC1bits.TRGSRC2 = 0b01100;   // ADC channels AN4 and AN5 have conversion triggered by Timer 1 period

    /* Set Analog Pins */
    ADPCFGbits.PCFG4 = 0;           // Select AN4 as analog pin
    ADPCFGbits.PCFG5 = 0;           // Select AN5 as analog pin 
    
    /* Handle Interrupt initialization */
    IPC28bits.ADCP2IP = 0x01;       // Set ADC Pair 2 Interrupt Priority (Level 2)
    IFS7bits.ADCP2IF = 0;           // Clear ADC Pair 2 Interrupt Flag
    IEC7bits.ADCP2IE = 1;           // Enable ADC Pair 2 Interrupt
    

    
    /* OTHER ADC SETUP */
    ADCPC0bits.TRGSRC1 = 0b11111;   // ADC channels AN2 and AN3 have conversion triggered by Timer 2 period
    
    /* Set Analog Pins */
    ADPCFGbits.PCFG2 = 0;           // Select AN2 as analog pin
    ADPCFGbits.PCFG3 = 0;           // Select AN3 as analog pin
    
    /* Handle Interrupt initialization */
    IPC27bits.ADCP1IP = 0x02;       // Set ADC Pair 1 Interrupt Priority
    IFS6bits.ADCP1IF= 0;            // Clear ADC Pair 1 Interrupt Flag
    IEC6bits.ADCP1IE = 1;           // Enable ADC Pair 1 Interrupt
    
    /* Turn on ADC Module */
    ADCONbits.ADON = 1;

}

void init_PWM(void) {
    
    PTCONbits.PTEN = 0;             // Disable PWM module
    
    /* Configure Clock Prescaler and Period */
    PTCON2 = 5;                     // Scale down clock by 2^(this number)
    PTPER = 2305;                   // PWM period value (must match with our clocks)
    /* Old was PTCON2=5, PTPER=2305 */
    /* Old was 163, 182 */
    
    /* PWM1 Configuration */
    IOCON1bits.PENH = 1;            // PWM1H is controlled by PWM module (formerly zero to be GPIO)
    IOCON1bits.PENL = 1;            // PWM1L is controlled by PWM module
    IOCON1bits.PMOD = 0b11;            // Select independent output PWM mode
    PWMCON1bits.ITB = 0;            // No independent timebase mode
    PWMCON1bits.MDCS = 0;           // PDCx and SDCx provide duty cycle information
    PDC1 = 0;                       // Initial duty cycle for PWMxH
    SDC1 = 0;                       // Initial duty cycle for PWMxL
    
    /* PWM2 Configuration */
    IOCON2bits.PENH = 1;            // PWM2H is controlled by PWM module (formerly zero to be GPIO)
    IOCON2bits.PENL = 1;            // PWM2L is controlled by PWM module
    IOCON2bits.PMOD = 0b11;         // Select independent output PWM mode (important)
    PWMCON2bits.ITB = 0;            // No independent timebase mode
    PWMCON2bits.MDCS = 0;           // PDCx and SDCx provide duty cycle information
    PDC2 = 0;                       // Initial duty cycle for PWMxH
    SDC2 = 0;                       // Initial duty cycle for PWMxL
    
    /* PWM3 Configuration */
    IOCON3bits.PENH = 1;            // PWM2H is controlled by PWM module (formerly zero to be GPIO)
    IOCON3bits.PENL = 1;            // PWM2L is controlled by PWM module
    IOCON3bits.PMOD = 0b11;         // Select independent output PWM mode (important)
    PWMCON3bits.ITB = 0;            // No independent timebase mode
    PWMCON3bits.MDCS = 0;           // PDCx and SDCx provide duty cycle information
    PDC3 = 0;                       // Initial duty cycle for PWMxH
    SDC3 = 0;                       // Initial duty cycle for PWMxL

    /* Enable PWM Module */
    PTCONbits.PTEN = 1;             // Enable PWM module
 
}

void init_UART(void){
        
    /* Set Configuration Bits */
    U2MODEbits.USIDL = 0;       // Continue in idle mode
    U2MODEbits.IREN = 0;        // IrDA encoder and decoder are disabled
    U2MODEbits.RTSMD = 0;       // U2RTS in Flow Control mode
    U2MODEbits.UEN = 0b00;      // U2TX and U2RX pins are enabled and used; no flow control pins
    U2MODEbits.WAKE = 0;        // Wake-up is disabled
    U2MODEbits.LPBACK = 0;      // Loopback mode is disabled
    U2MODEbits.ABAUD = 0;       // Autobaud is off
    U2MODEbits.URXINV = 0;      // U2RX idle state is '1'
    U2MODEbits.BRGH = 1;        // High speed mode (BRG generates 4 clocks per bit period)
    U2MODEbits.PDSEL = 0b00;    // 8-bit data, no parity
    U2MODEbits.STSEL = 0;       // 1 stop bit
    U2STAbits.URXISEL = 0b00;   // Interrupt flag bit is set when a character is received
    U2MODEbits.UARTEN = 1;      // Enable UART RX/TX
    U2STAbits.UTXEN = 1;        // Enable the transmitter
    
    /* Configure Baud Rate */
    int brg = (FCY/BAUDRATE/4)-1;   // Usual equation for high-speed mode (BRGH=1)
    brg = 85;                       // round up, since it's closer
    U2BRG = brg;                    // Store the value in the register
    
    /* Set up UART Interrupts*/
    IPC7bits.U2RXIP = 0b01;         // Set U2RX interrupt priority
    IFS1bits.U2RXIF = 0;            // Clear the U2RX interrupt flag
    IEC1bits.U2RXIE = 1;            // Enable U2RX interrupt
    
}

void send_UART(char c){
    // wait for transmit buffer to be empty
    while (U2STAbits.UTXBF);
    // write character to transmit register
    U2TXREG = c;
}

unsigned char receive_UART(void){
    // Formerly uint8_t
    while (1){
        if (U2STAbits.FERR == 1){
            continue;
        }
        
        if (U2STAbits.OERR == 1){
            U2STAbits.OERR = 0;
            continue;
        }
        
        if (U2STAbits.URXDA == 1){
            return U2RXREG;
        }
    }
}

double decode_UART(void) {
    
    /* Initialize placeholders */
    char results[5];
    double result = 0;
    
    /* Initialize counting variable */
    int count = 0;
    
    /* Set while loop to end at SPACE */
    while (count < 5){
        results[count++] = receive_UART();
    }
    
    receive_UART();                     // handle the space
        
    /* Determine Result */
    result = atof(results);

    return result;
    
}

void init_SPI(void) {
    
    SPI1CON1bits.DISSCK = 0;                // Internal serial clock is enabled
    SPI1CON1bits.DISSDO = 0;                // SDO1 pin is controlled by the SPI module
    SPI1CON1bits.MODE16 = 0;                // 8-bit communication
    SPI1CON1bits.MSTEN = 1;                 // Master mode enabled
    SPI1CON1bits.SMP = 0;                   // Input data is sampled at the middle of data output time
    SPI1CON1bits.SSEN = 0;                  // SS1 is controlled by the PORT function
    SPI1CON1bits.CKE = 1;                   // Serial output data changes on transition
                                            // from idle clock state to active clock state
    SPI1CON1bits.CKP = 0;                   // Idle state for clock is low level; active high
    SPI1CON1bits.PPRE = 0b11;               // Primary prescaler 64:1
    SPI1CON1bits.SPRE = 0b111;              // Secondary prescaler 8:1
    SPI1STATbits.SPIEN = 1;                 // Enable SPI module
    
}

void stopMotors(void) {
    SDC1 = 0;
    PDC1 = 0;
    SDC2 = 0;
    PDC2 = 0;
    SDC3 = 0;
    PDC3 = 0;
}

int motorControl(double measured, double target){
    
    double difference;
    int div;
    int string;
    int flat = 0;
    
    if (measured < 50 || measured > 340 || target < 50){
        stopMotors();
        return 8;
    }
    
    /* Determine String */
    if (target < 87 && target > 61){
        string = 1;
    }
    else if (target < 116 && target > 87){
        string = 2;
    }
    else if (target < 155 && target > 116){
        string = 3;
    }
    else if (target < 207 && target > 155){
        string = 4;
    }
    else if (target < 261 && target > 207){
        string = 5;
    }
    else if (target < 330 && target > 261) {
        string = 6;
    }
    else {
        return 8;
    }
    
    if (measured <= target-threshold){
//        if (string != 1)
            tuningDisplay(string-1, 0);
        flat = 1;
    }
    else if (measured > target+threshold){
//        if (string != 1)
            tuningDisplay(string-1, 2);
        flat = 0;
    }
    else {
        tuningDisplay(string-1, 1);
        stopMotors();
        return 8;
    }
    
    difference = abs(measured - target);
    if (difference > 14){
        div = 1;
    }
    else if (difference > 9){
        div = 2;
    }
    else if (difference > 4){
        div = 4;
    }
    else {
        div = 8;
    }
    
    switch (string) {
        
        case 1:
            if (flat == 1) {
                s1Up();
            }
            else {
                s1Down();
            }
            break;
            
        case 2:
            if (flat == 1) {
                s2Up();
//                s1Up();
            }
            else {
                s2Down();
//                s1Down();
            }
            break;
            
        case 3:
            if (flat == 1) {
                s3Up();
//                s1Up();
            }
            else {
                s3Down();
//                s1Down();
            }
            break;
            
        case 4:
            if (flat == 1) {
                s4Up();
//                s1Up();
            }
            else {
                s4Down();
//                s1Down();
            }
            break;
            
        case 5:
            if (flat == 1) {
                s5Up();
//                s1Up();
            }
            else {
                s5Down();
//                s1Down();
            }
            break;
            
        case 6:
            if (flat == 1) {
                s6Up();
//                s1Up();
            }
            else {
                s6Down();
//                s1Down();
            }
            break;
            
        default:
            stopMotors();
        
    }
    
    
    return div;
}

int masterControl(uint16_t *fft, int numPoints, double *freqs){
    
    int div;
    int m2 = 0;
    int m3 = 0;
    int k, j;
    int ind[SORT] = {0};
    uint16_t mag[SORT] = {0};
    int startpoint = 50/ADC_RATE*ADC_BUFFER_SIZE;

    /* Find max points */
    for (k=startpoint; k<numPoints; k++){
        if (fft[k] > mag[0]) {
            for (j=1; j<SORT; j++){
                ind[SORT-j] = ind[SORT-j-1];
                mag[SORT-j] = mag[SORT-j-1];
            }
            ind[0] = k;
            mag[0] = fft[k];
        }
        else if (fft[k] > mag[1]) {
            for (j=1; j<SORT-1; j++){
                ind[SORT-j] = ind[SORT-j-1];
                mag[SORT-j] = mag[SORT-j-1];
            }
            ind[1] = k;
            mag[1] = fft[k];
        }
        else if (fft[k] > mag[2]) {
            for (j=1; j<SORT-2; j++){
                ind[SORT-j] = ind[SORT-j-1];
                mag[SORT-j] = mag[SORT-j-1];                
            }
            ind[2] = k;
            mag[2] = fft[k];
        }
        else if (fft[k] > mag[3]) {
            for (j=1; j<SORT-3; j++){
                ind[SORT-j] = ind[SORT-j-1];
                mag[SORT-j] = mag[SORT-j-1];                
            }
            ind[3] = k;
            mag[3] = fft[k];
        }
        else if (fft[k] > mag[4]) {
            for (j=1; j<SORT-4; j++){
                ind[SORT-j] = ind[SORT-j-1];
                mag[SORT-j] = mag[SORT-j-1];                
            }
            ind[4] = k;
            mag[4] = fft[k];
        }
        else if (fft[k] > mag[5]) {
            ind[5] = k;
            mag[5] = fft[k];
        }
    }
    
    /* Calculate frequencies */
    for (k=0; k<SORT; k++){
        peaks[k] = ind[k]*ADC_RATE/ADC_BUFFER_SIZE;
    }
    
    /* Find multiplicities */
    for (k=1; k<SORT; k++){
        if ((ind[k]*2)+4 > ind[0] && (ind[k]*2)-4 < ind[0]){
            m2 = 1;
        }
        else if ((ind[k]*3)+4 > ind[0] && (ind[k]*3)-4 < ind[0]){
            m3 = 1;
        }
    }
    
    /* Set fundamental value */
    if (m2 == 1) {
        fundamental = peaks[0]/2;
    }
    else if (m3 == 1){
        fundamental = peaks[0] / 3;
    }
    else {
        fundamental = peaks[0];
    }

    
    if (fundamental < 87.31-1.5*threshold && fundamental > 61.74-1.5*threshold){
        mode = 1;
    }
    else if (fundamental < 116.5-1.5*threshold && fundamental > 87.31-1.5*threshold){
        mode = 2;
    }
    else if (fundamental < 155.6-1.5*threshold && fundamental > 116.5-1.5*threshold && m2 == 0){
        mode = 3;
    }
    else if (fundamental < 207.7-1.5*threshold && fundamental > 155.6-1.5*threshold){
        mode = 4;
    }
    else if (fundamental < 261.6-1.5*threshold && fundamental > 207.7-1.5*threshold){
        mode = 5;
    }
    else if (fundamental < 329.6+4*threshold && fundamental > 261.6-1.5*threshold){
        mode = 6;
    }
    else if (fundamental > 329.6+2*threshold) {
        mode = 7;
    }
    else {
        mode = 0;
    }

    /* Error Catching */
    if (last_fundamental != 0 && abs(fundamental-last_fundamental)>25){
        mode = 7;
    }
    
    /* Magnitude Threshold */
    if (mag[0] < 50){
        mode = 0;
    }    
    
    switch (mode) {
        
        case 1:
            div = motorControl(fundamental, freqs[0]);
            break;
            
        case 2:
            div = motorControl(fundamental, freqs[1]);
            break;
            
        case 3:
            div = motorControl(fundamental, freqs[2]);
            break;
            
        case 4:
            div = motorControl(fundamental, freqs[3]);
            break;
            
        case 5:
            div = motorControl(fundamental, freqs[4]);
            break;
            
        case 6:
            div = motorControl(fundamental, freqs[5]);
            break;
            
        case 0:
            div = motorControl(0, 0);
            last_fundamental = 0;
            return div;
            
        default:
            div = motorControl(0, 0);
            return div;
            
    }
    
    last_fundamental = fundamental;
    
    return div;
    
    
}

void init_timer2(void) {
    
    /* Timer 2 Configuration */
    T2CONbits.TON = 0;                  // Turn off Timer 2
    T2CONbits.T32 = 0;                  // 16-bit mode
    T2CONbits.TCS = 0;                  // Internal clock selected
    T2CONbits.TGATE = 0;
    T2CONbits.TCKPS = 0b10;             // Clock prescaler 1:1
    
    /* Timer 2 Value Setting*/
    TMR2 = 0;                           // Reset Timer 2 Value
    PR2 = 15475;                    // Timer 2 Period Match (around 40 Hz)
    
    /* Timer 2 Interrupt Handling */
    IPC1bits.T2IP = 0x01;               // Set Timer 2 Interrupt Priority
    IFS0bits.T2IF = 0;                  // Clear Timer 2 Interrupt Flag
    IEC0bits.T2IE = 0;                  // Enable Timer 2 Interrupt
    
    /* Timer 2 Activation */
    T2CONbits.TON = 1;                  // Turn on Timer 2
    
}

void resetDisplay(void) {
    
  int j;
  for(j=0;j<=5;j++) {
      noteSpacing[j] = 25 + j*spacing;
  }
      
  tft_fillScreen(ILI9340_BLACK);
  tft_setRotation(3);
//  tft_setRotation(1);

  //boxes
  int i;
  for(i=0; i<=5; i++) {
    tft_drawRect(12+i*spacing, 91, noteBoxSize, noteBoxSize, ILI9340_WHITE);
  }

  //notes in boxes
  updateDisplayText(1); 

  //save button
  tft_drawRect(235, 210, 73, 30, ILI9340_WHITE);
  tft_fillRect(236, 211, 71, 28, ILI9340_BLACK);
  tft_setCursor(248, 217);
  tft_setTextSize(2);
  tft_writeString("TUNE");
}

void updateDisplayText(int noteRst) {
    int i;
    if (noteRst == 0) { //reset
    //blue out box
    tft_fillRect(12+selectedString*spacing+1,92,noteBoxSize-2,noteBoxSize-2,ILI9340_BLUE);
  } else { //select a note
    //black out boxes
      for(i=0; i<=5; i++) {
        tft_fillRect(12+i*spacing+1, 92, noteBoxSize-2, noteBoxSize-2, ILI9340_BLACK);
        tft_drawRect(12+i*spacing, 91, noteBoxSize, noteBoxSize, ILI9340_WHITE);
      }
  }
  //add new text
  tft_setTextColor(ILI9340_WHITE);
  tft_setTextSize(4);
  int start_y = 100;
  
  for (i=0; i<=5; i++) {
    tft_setCursor(noteSpacing[i], start_y); 
    tft_setTextSize(4);
    //tft_writeString(fullNotesArray[setNotesArray[i]]); //write notes into boxes
    tft_write(fullNotesArray[setNotesArray[i]]); //write notes into boxes
    
    if (isSharp[i] == 1) {
      tft_setCursor(noteSpacing[i]+25,start_y);
      tft_setTextSize(1.2);
      tft_writeString("#"); //add sharps
    }
  }
}

void selectNote(void) {
  //resetDisplay();
int i;    
  //clear old selection
for (i=0; i<=5; i++) {
    if (i!=selectedString) {
      tft_fillRect(10+i*spacing,80-noteBoxSize,noteBoxSize+3,noteBoxSize+3,ILI9340_BLACK);
      tft_fillRect(10+i*spacing,91+noteBoxSize+8,noteBoxSize+3,noteBoxSize+3,ILI9340_BLACK);
      tft_fillRect(12+i*spacing, 91, noteBoxSize, noteBoxSize, ILI9340_BLACK);
      tft_drawRect(12+i*spacing, 91, noteBoxSize, noteBoxSize, ILI9340_WHITE);
    }
  }
  //update new note selection
  tft_fillRect(12+selectedString*spacing, 91, noteBoxSize, noteBoxSize, ILI9340_BLUE);
  tft_fillTriangle(12+selectedString*spacing, 81, 12+selectedString*spacing+noteBoxSize, 81, 12+selectedString*spacing+0.5*noteBoxSize,81-noteBoxSize,ILI9340_WHITE);
  tft_fillTriangle(12+selectedString*spacing, 91+noteBoxSize+10, 12+selectedString*spacing+noteBoxSize, 91+noteBoxSize+10, 12+selectedString*spacing+0.5*noteBoxSize,91+(2*noteBoxSize)+10,ILI9340_WHITE);

  updateDisplayText(0);
}

void rangeMapping(int input_x, int input_y) {
  //double limits_touch[4] = {316, 0, 6, 246};
  //int limits_disp[4] = {0, 240, 0, 320};
  float dispMaxX = 320.0;
  float dispMaxY = 240.0;
  int touchX, touchY, dispX, dispY;

  // map raw touch data to touch screen coordinates 
  touchX = 1.0*(input_x-TS_MINX)/(TS_MAXX-TS_MINX)*dispMaxY;
  touchY = 1.0*(input_y-TS_MINY)/(TS_MAXY-TS_MINY)*dispMaxX;
  
  // flip touch coordinates to map to disp coordinates
  dispX = dispMaxX - touchY;
  dispY = touchX;
//  dispX = touchY;
//  dispY = dispMaxY - touchX;

  touchLoc[0] = dispX;
  touchLoc[1] = dispY;
  return;
}

void startTuning(void) {

  //change tuning flag variable
  isTuning = 1;

  //reset note boxes to black
  updateDisplayText(1);
  
  //light up tune button
  tft_fillRect(236, 211, 71, 28, ILI9340_BLUE);
  tft_setCursor(248, 217);
  tft_setTextSize(2);
  tft_writeString("TUNE");

  //add "STOP" button
  tft_drawRect(25, 210, 73, 30, ILI9340_WHITE);
  tft_fillRect(26, 211, 71, 28, ILI9340_BLACK);
  tft_setCursor(38, 217);
  tft_setTextSize(2);
  tft_writeString("STOP");

  //remove triangles
  tft_fillTriangle(12+selectedString*spacing, 81, 12+selectedString*spacing+noteBoxSize, 81, 12+selectedString*spacing+0.5*noteBoxSize,81-noteBoxSize,ILI9340_BLACK);
  tft_fillTriangle(12+selectedString*spacing, 91+noteBoxSize+10, 12+selectedString*spacing+noteBoxSize, 91+noteBoxSize+10, 12+selectedString*spacing+0.5*noteBoxSize,91+(2*noteBoxSize)+10,ILI9340_BLACK);

  
//  //display tuning progress
//  int tuningStringNum = 0;
//  tuningDisplay(tuningStringNum);
}

void stopTuning(void) {
  //change tuning flag variable
  isTuning = 0;

  //reset note boxes to black
  selectedString = 0;
  selectNote();
  
  //remove stop button
  tft_fillRect(25, 210, 73, 30, ILI9340_BLACK);

  //"unpress" tune button
  tft_fillRect(236, 211, 71, 28, ILI9340_BLACK);
  tft_setCursor(248, 217);
  tft_setTextSize(2);
  tft_writeString("TUNE");

}

void tuningDisplay(int tuningStringNum, int mode) {

    if (tuningStringNum>=0 && tuningStringNum <=5){
      switch(mode) { //check status and display accordingly
        case 0: //flat
          tft_setCursor(noteSpacing[tuningStringNum], 100); 
          tft_setTextSize(4);
          tft_write(fullNotesArray[setNotesArray[tuningStringNum]]);
          tft_fillCircle(12+tuningStringNum*spacing+0.5*noteBoxSize,65,8,ILI9340_BLACK);
          tft_fillCircle(12+tuningStringNum*spacing+0.5*noteBoxSize,95+noteBoxSize+21,8,ILI9340_RED);
          break;
        case 1: //tuned
          tft_fillCircle(12+tuningStringNum*spacing+0.5*noteBoxSize,65,8,ILI9340_BLACK);
          tft_fillCircle(12+tuningStringNum*spacing+0.5*noteBoxSize,95+noteBoxSize+21,8,ILI9340_BLACK);
          tft_setTextColor(ILI9340_GREEN);
          tft_setCursor(noteSpacing[tuningStringNum], 100); 
          tft_setTextSize(4);
          tft_write(fullNotesArray[setNotesArray[tuningStringNum]]);
          tft_setTextColor(ILI9340_WHITE);
          break;
        case 2: //sharp
          tft_setCursor(noteSpacing[tuningStringNum], 100); 
          tft_setTextSize(4);
          tft_write(fullNotesArray[setNotesArray[tuningStringNum]]);
          tft_fillCircle(12+tuningStringNum*spacing+0.5*noteBoxSize,95+noteBoxSize+21,8,ILI9340_BLACK);  
          tft_fillCircle(12+tuningStringNum*spacing+0.5*noteBoxSize,65,8,ILI9340_RED);
          break;
        default:
          tft_fillTriangle(12+selectedString*spacing, 81, 12+selectedString*spacing+noteBoxSize, 81, 12+selectedString*spacing+0.5*noteBoxSize,81-noteBoxSize,ILI9340_BLACK);
          tft_fillTriangle(12+selectedString*spacing, 91+noteBoxSize+10, 12+selectedString*spacing+noteBoxSize, 91+noteBoxSize+10, 12+selectedString*spacing+0.5*noteBoxSize,91+(2*noteBoxSize)+10,ILI9340_BLACK);
          tft_fillRect(10+tuningStringNum*spacing,80-noteBoxSize,noteBoxSize+3,noteBoxSize+3,ILI9340_BLACK);
          tft_setTextColor(ILI9340_WHITE);
          tft_setCursor(noteSpacing[tuningStringNum], 100); 
          tft_setTextSize(4);
          tft_write(fullNotesArray[setNotesArray[tuningStringNum]]);
      }
      
    }
    
        tft_writecommand(ILI9340_INVOFF);
        tft_writecommand(ILI9340_DISPON);

}

void freqPicker(void) {
   double frequencies0[6] = {82.41, 77.88, 73.42, 69.30, 65.41, 61.74};
   double frequencies1[5] = {110.0, 103.8, 98.01, 92.51, 87.31};
   double frequencies2[5] = {146.8, 138.6, 130.8, 123.5, 116.5};
   double frequencies3[5] = {196.0, 185.0, 174.6, 164.8, 155.6};
   double frequencies4[4] = {246.9, 233.1, 220.0, 207.7};
   double frequencies5[5] = {329.6, 311.1, 293.7, 277.2, 261.6};
   int i=0;
   
    if (setNotesArray[i]==4 && isSharp[i]==0)
        frequencies[i] = frequencies0[0];
    else if (setNotesArray[i]==3 && isSharp[i]==1)
        frequencies[i] = frequencies0[1];
    else if (setNotesArray[i]==3 && isSharp[i]==0)
        frequencies[i] = frequencies0[2];
    else if (setNotesArray[i]==2 && isSharp[i]==1)
        frequencies[i] = frequencies0[3];
    else if (setNotesArray[i]==2 && isSharp[i]==0)
        frequencies[i] = frequencies0[4];
    else if (setNotesArray[i]==1)
        frequencies[i] = frequencies0[5];
    else
        frequencies[i] = frequencies0[0];
      
    i=1;
    if (setNotesArray[i]==0 && isSharp[i]==0)
        frequencies[i] = frequencies1[0];
    else if (setNotesArray[i]==6 && isSharp[i]==1)
        frequencies[i] = frequencies1[1];
    else if (setNotesArray[i]==6 && isSharp[i]==0)
        frequencies[i] = frequencies1[2];
    else if (setNotesArray[i]==5 && isSharp[i]==1)
        frequencies[i] = frequencies1[3];
    else if (setNotesArray[i]==5 && isSharp[i]==0)
        frequencies[i] = frequencies1[4];
    else
        frequencies[i] = frequencies1[0];
      
    i=2;
    if (setNotesArray[i]==3 && isSharp[i]==0)
        frequencies[i] = frequencies2[0];
    else if (setNotesArray[i]==2 && isSharp[i]==1)
        frequencies[i] = frequencies2[1];
    else if (setNotesArray[i]==2 && isSharp[i]==0)
        frequencies[i] = frequencies2[2];
    else if (setNotesArray[i]==1)
        frequencies[i] = frequencies2[3];
    else if (setNotesArray[i]==0 && isSharp[i]==1)
        frequencies[i] = frequencies2[4];
    else
        frequencies[i] = frequencies2[0];              
      
    i=3;
    if (setNotesArray[i]==6 && isSharp[i]==0)
        frequencies[i] = frequencies3[0];
    else if (setNotesArray[i]==5 && isSharp[i]==1)
        frequencies[i] = frequencies3[1];
    else if (setNotesArray[i]==5 && isSharp[i]==0)
        frequencies[i] = frequencies3[2];
    else if (setNotesArray[i]==4)
        frequencies[i] = frequencies3[3];
    else if (setNotesArray[i]==3 && isSharp[i]==1)
        frequencies[i] = frequencies3[4];
    else
        frequencies[i] = frequencies3[0];             
           
    i=4;
    if (setNotesArray[i]==1)
        frequencies[i] = frequencies4[0];
    else if (setNotesArray[i]==0 && isSharp[i]==1)
        frequencies[i] = frequencies4[1];
    else if (setNotesArray[i]==0 && isSharp[i]==0)
        frequencies[i] = frequencies4[2];
    else if (setNotesArray[i]==6 && isSharp[i]==1)
        frequencies[i] = frequencies4[3];
    else
        frequencies[i] = frequencies4[0];
    
    i=5;
    if (setNotesArray[i]==4 && isSharp[i]==0)
        frequencies[i] = frequencies5[0];
    else if (setNotesArray[i]==3 && isSharp[i]==1)
        frequencies[i] = frequencies5[1];
    else if (setNotesArray[i]==3 && isSharp[i]==0)
        frequencies[i] = frequencies5[2];
    else if (setNotesArray[i]==2 && isSharp[i]==1)
        frequencies[i] = frequencies5[3];
    else if (setNotesArray[i]==2 && isSharp[i]==0)
        frequencies[i] = frequencies5[4];
    else
        frequencies[i] = frequencies5[0];   
    

}

void updateScreenMemoryArray(void) {
    
   double frequencies0[6] = {82.41, 77.88, 73.42, 69.30, 65.41, 61.74};
   double frequencies1[5] = {110.0, 103.8, 98.01, 92.51, 87.31};
   double frequencies2[5] = {146.8, 138.6, 130.8, 123.5, 116.5};
   double frequencies3[5] = {196.0, 185.0, 174.6, 164.8, 155.6};
   double frequencies4[4] = {246.9, 233.1, 220.0, 207.7};
   double frequencies5[5] = {329.6, 311.1, 293.7, 277.2, 261.6};
   
    
    int i;
    
    for (i=0; i<6; i++){
        if (abs(frequencies[0]-frequencies0[i]) < 1) {
            switch (i){
                case 0:
                    setNotesArray[0] = 4;
                    isSharp[0] = 0;
                    break;
                case 1:
                    setNotesArray[0] = 3;
                    isSharp[0] = 1;
                    break;
                case 2:
                    setNotesArray[0] = 3;
                    isSharp[0] = 0;
                    break;
                case 3:
                    setNotesArray[0] = 2;
                    isSharp[0] = 1;
                    break;
                case 4:
                    setNotesArray[0] = 2;
                    isSharp[0] = 0;
                    break;
                case 5:
                    setNotesArray[0] = 1;
                    isSharp[0] = 0;
                    break;
            }
        }
    }
    
    for (i=0; i<5; i++){
        if (abs(frequencies[1]-frequencies1[i]) < 1) {
            switch (i){
                case 0:
                    setNotesArray[1] = 0;
                    isSharp[1] = 0;
                    break;
                case 1:
                    setNotesArray[1] = 6;
                    isSharp[1] = 1;
                    break;
                case 2:
                    setNotesArray[1] = 6;
                    isSharp[1] = 0;
                    break;
                case 3:
                    setNotesArray[1] = 5;
                    isSharp[1] = 1;
                    break;
                case 4:
                    setNotesArray[1] = 5;
                    isSharp[1] = 0;
                    break;
            }
        }
    }
    
    for (i=0; i<5; i++){
        if (abs(frequencies[2]-frequencies2[i]) < 1) {
            switch (i){
                case 0:
                    setNotesArray[2] = 3;
                    isSharp[2] = 0;
                    break;
                case 1:
                    setNotesArray[2] = 2;
                    isSharp[2] = 1;
                    break;
                case 2:
                    setNotesArray[2] = 2;
                    isSharp[2] = 0;
                    break;
                case 3:
                    setNotesArray[2] = 1;
                    isSharp[2] = 0;
                    break;
                case 4:
                    setNotesArray[2] = 0;
                    isSharp[2] = 1;
                    break;
            }
        }
    }
    
    for (i=0; i<5; i++){
        if (abs(frequencies[3]-frequencies3[i]) < 1) {
            switch (i){
                case 0:
                    setNotesArray[3] = 6;
                    isSharp[3] = 0;
                    break;
                case 1:
                    setNotesArray[3] = 5;
                    isSharp[3] = 1;
                    break;
                case 2:
                    setNotesArray[3] = 5;
                    isSharp[3] = 0;
                    break;
                case 3:
                    setNotesArray[3] = 4;
                    isSharp[3] = 0;
                    break;
                case 4:
                    setNotesArray[3] = 3;
                    isSharp[3] = 1;
                    break;
            }
        }
    }
    
    for (i=0; i<4; i++){
        if (abs(frequencies[4]-frequencies4[i]) < 1) {
            switch (i){
                case 0:
                    setNotesArray[4] = 1;
                    isSharp[4] = 0;
                    break;
                case 1:
                    setNotesArray[4] = 0;
                    isSharp[4] = 1;
                    break;
                case 2:
                    setNotesArray[4] = 0;
                    isSharp[4] = 0;
                    break;
                case 3:
                    setNotesArray[4] = 6;
                    isSharp[4] = 1;
                    break;
            }
        }
    }
    
    for (i=0; i<5; i++){
        if (abs(frequencies[5]-frequencies5[i]) < 1) {
            switch (i){
                case 0:
                    setNotesArray[5] = 4;
                    isSharp[5] = 0;
                    break;
                case 1:
                    setNotesArray[5] = 3;
                    isSharp[5] = 1;
                    break;
                case 2:
                    setNotesArray[5] = 3;
                    isSharp[5] = 0;
                    break;
                case 3:
                    setNotesArray[5] = 2;
                    isSharp[5] = 1;
                    break;
                case 4:
                    setNotesArray[5] = 2;
                    isSharp[5] = 0;
                    break;
            }
        }
    }
    
}